package test.dao.impl;

import dao.impl.TeacherDAOImpl;
import model.Teacher;
import org.junit.After;
import org.junit.Before;
import org.junit.Test;
import util.DatabaseUtil;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.SQLException;

import static org.junit.Assert.*;

/**
 * TeacherDAOImpl的单元测试类
 * 测试教师数据访问层的各项功能
 */
public class TeacherDAOImplTest {
    
    private TeacherDAOImpl teacherDAO;
    private static final String TEST_TEACHER_ID = "T2024001";
    
    @Before
    public void setUp() throws SQLException {
        teacherDAO = new TeacherDAOImpl();
        // 清理测试数据
        cleanTestData();
    }
    
    @After
    public void tearDown() throws SQLException {
        cleanTestData();
    }
    
    /**
     * 清理测试数据
     */
    private void cleanTestData() throws SQLException {
        try (Connection conn = DatabaseUtil.getConnection();
             PreparedStatement stmt = conn.prepareStatement("DELETE FROM teachers WHERE teacher_id = ?")) {
            stmt.setString(1, TEST_TEACHER_ID);
            stmt.executeUpdate();
        }
    }
    
    /**
     * 创建测试用的Teacher对象
     */
    private Teacher createTestTeacher() {
        Teacher teacher = new Teacher();
        teacher.setTeacherId(TEST_TEACHER_ID);
        teacher.setName("李四");
        teacher.setDepartment("计算机科学系");
        teacher.setContact("13900139000");
        teacher.setPassword("password123");
        return teacher;
    }
    
    @Test
    public void testInsert() {
        // 准备测试数据
        Teacher teacher = createTestTeacher();
        
        // 执行插入操作
        int result = teacherDAO.insert(teacher);
        
        // 验证结果
        assertEquals(1, result);
        assertTrue(teacher.getId() > 0);
        
        // 验证数据是否正确插入
        Teacher foundTeacher = teacherDAO.findByTeacherId(TEST_TEACHER_ID);
        assertNotNull(foundTeacher);
        assertEquals(teacher.getName(), foundTeacher.getName());
        assertEquals(teacher.getDepartment(), foundTeacher.getDepartment());
        assertEquals(teacher.getContact(), foundTeacher.getContact());
    }
    
    @Test
    public void testFindByTeacherId() {
        // 准备测试数据
        Teacher teacher = createTestTeacher();
        teacherDAO.insert(teacher);
        
        // 执行查询操作
        Teacher foundTeacher = teacherDAO.findByTeacherId(TEST_TEACHER_ID);
        
        // 验证结果
        assertNotNull(foundTeacher);
        assertEquals(TEST_TEACHER_ID, foundTeacher.getTeacherId());
        assertEquals(teacher.getName(), foundTeacher.getName());
    }
    
    @Test
    public void testFindByTeacherIdNotFound() {
        // 查询不存在的工号
        Teacher foundTeacher = teacherDAO.findByTeacherId("nonexistent");
        
        // 验证结果
        assertNull(foundTeacher);
    }
    
    @Test
    public void testInsertDuplicate() {
        // 准备测试数据
        Teacher teacher1 = createTestTeacher();
        Teacher teacher2 = createTestTeacher();
        
        // 第一次插入
        int result1 = teacherDAO.insert(teacher1);
        assertEquals(1, result1);
        
        // 尝试插入重复的工号
        int result2 = teacherDAO.insert(teacher2);
        assertEquals(0, result2);
    }
}
